<?php
// $Id: temporary_invitation_api.inc,v 1.1.2.7 2007/05/16 14:28:47 jpetso Exp $
/**
 * @file
 * Temporary Invitation - Invite guests for a limited timespan.
 * This file contains the "reusable" parts of the Temporary Invitation module.
 *
 * Copyright 2007 by Jakob Petsovits <jpetso@gmx.at>
 * Copyright 2006 by D R Pratten <http://www.davidpratten.com>
 * Distributed under the GNU General Public Licence version 2 or higher,
 * as published by the FSF on http://www.gnu.org/copyleft/gpl.html
 */


/**
 * Invokes hook_temporary_invitation() for all modules that implemented it.
 *
 * @param $op
 *   What kind of action is being performed. Possible values:
 *   - "insert": The invitation has been created and is readily accessible.
 *   - "delete": The invitation is about to be deleted from the database, either
 *               because it has expired or because the host user has been
 *               deleted.
 * @param $invitation
 *   An invitation object, as returned by temporary_invitation_load().
 * @param $passcode
 *   If $op == "insert", this contains the passcode as plaintext.
 *   For all other hook invocations this is NULL.
 */
function temporary_invitation_invoke_all($op, $invitation, $passcode = NULL) {
  module_invoke_all('temporary_invitation', $op, $invitation, $passcode);
}


// array_fill_keys() (which is used in here) is only available in PHP 5.2
// or higher, so define it here for previous versions. This implementation
// depends on array_combine() which still requires PHP 5 or higher,
// so define that one as well.

if (!function_exists("array_combine")) {
  function array_combine($array1, $array2)
  {
    for ($i = 0; $i < count($array1); $i++) {
      $return_array[$array1[$i]] = $array2[$i];
    }
    if (isset($return_array)) {
      return $return_array;
    }
    return FALSE;
  }
}

if (!function_exists("array_fill_keys")) {
  function array_fill_keys($keys, $value) {
    if (count($keys) == 0) {
      return array();
    }
    return array_combine($keys, array_fill(0, count($keys), $value));
  }
}



/**
 * Retrieve all invitation objects for a given host user.
 *
 * @param $host_uid
 *   User ID of the user whose invitations shall be retrieved.
 *
 * @returns  An array of invitations for the host user. The invitations
 *           themselves are objects with the attributes 'host_uid',
 *           'invited_uid', 'name', and 'ticket' (where 'ticket' is a
 *           complete ticket object as returned by loginticket_load()).
 */
function temporary_invitation_load_array($host_uid) {
  $result = db_query("SELECT * FROM {temporary_invitation}
                      WHERE host_uid = '%d'", $host_uid);
  $invitations = array();

  while ($invitation = db_fetch_object($result)) {
    $invitation->ticket = loginticket_load(
      'temporary_invitation', array('uid' => $invitation->invited_uid)
    );
    $invitations[] = $invitation;
  }
  return $invitations;
}

/**
 * Retrieve the invitation object for a given invited user.
 *
 * @param $invited_uid
 *   User ID of the user that has been invited.
 *
 * @returns  An invitation object with attributes named 'host_uid',
 *           'invited_uid', 'name', and 'ticket' (where 'ticket' is a
 *           complete ticket object as returned by loginticket_load()).
 */
function temporary_invitation_load($invited_uid) {
  $result = db_query("SELECT * FROM {temporary_invitation}
                      WHERE invited_uid = '%d'", $invited_uid);

  while ($invitation = db_fetch_object($result)) {
    $invitation->ticket = loginticket_load(
      'temporary_invitation', array('uid' => $invitation->invited_uid)
    );
    return $invitation;
  }
  return FALSE;
}

/**
 * Create a temporary invitation, that is, create a new user and relate this
 * user with the given host user, and then create a login ticket that
 * the generated user can use to login.
 * When the ticket is created, hook_temporary_invitation($op='insert') is
 * called (which in turn is used by Temporary Invitation to notify the user
 * by sending login code and the link to the login form.
 *
 * @param $host_user
 *   The user that manages the invitation.
 * @param $invitation_name
 *   Short description for the invitation.
 * @param $expiration_time
 *   A Unix timestamp specifying when the invitation will expire.
 * @param $invited_user_additions
 *   You can optionally specify an array with additional user info fields
 *   which will be passed on to user_save() for the newly created user.
 *   For example, passing array('mail' => 'user@example.com') would store
 *   the invited user's mail address in the 'mail' column of the {users} table.
 *
 * @return   An array, with the following values given for these keys:
 *   'invitation': An object containing the newly created invitation,
 *                 as returned by temporary_invitation_load().
 *   'passcode':   The passcode of the invitation as plaintext.
 */
function temporary_invitation_create($host_user, $invitation_name,
                                     $expiration_time, $invited_user_additions = array()) {
  $invited_user = temporary_invitation_create_user($host_user, $invited_user_additions);
  db_query("INSERT INTO {temporary_invitation} (host_uid, invited_uid, name)
            VALUES ('%d', '%d', '%s')",
           $host_user->uid, $invited_user->uid, $invitation_name);

  $passcode = loginticket_create('temporary_invitation', $invited_user, $expiration_time);
  $invitation = temporary_invitation_load($invited_user->uid);

  temporary_invitation_invoke_all('insert', $invitation, $passcode);

  return array(
    'invitation' => $invitation,
    'passcode' => $passcode,
  );
}

/**
 * Create a user with a random password and username.
 *
 * @param $host_user
 *   The user object of the user who invites the guest.
 * @param $invited_user_additions
 *   You can optionally specify an array with additional user info fields
 *   which will be passed on to user_save() for the newly created user.
 *   For example, passing array('mail' => 'user@example.com') would store
 *   the invited user's mail address in the 'mail' column of the {users} table.
 *
 * @return  The user object as delivered by user_save().
 */
function temporary_invitation_create_user($host_user, $invited_user_additions = array()) {
  do {
    $password = user_password(8);
    $username = t('Temporary user !name',
                  array('!name' => $password .'-'. $host_user->uid));
    $user = user_load(array('name' => $username));
  } while ($user); // loop until we have a username that doesn't exist

  $userinfo = array(
    'name' => $username,
    'pass' => $password,
    'status' => 1,
    'roles' => array_fill_keys(_temporary_invitation_get_user_roles_default(), 1),
  );
  $userinfo = array_merge($userinfo, $invited_user_additions);
  $user = user_save('', $userinfo);

  watchdog('user',
    t('New temporarily invited user: %username.', array('%username' => $username)),
    WATCHDOG_NOTICE,
    l(t('edit'), 'user/'. $user->uid .'/edit')
  );
  return $user;
}

/**
 * Delete the invitation for the given user that had been invited, and call
 * hook_temporary_invitation($op='delete', $invitation, NULL) before that.
 *
 * @param $host_uid
 *   User ID of the host user.
 * @param $invited_uid
 *   User ID of the invited user.
 * @param $goto_path
 *   Where to go after the invitation has been deleted.
 *   If NULL, drupal_goto() is not called, and you can continue execution.
 */
function temporary_invitation_delete($host_uid, $invited_uid, $goto_path = NULL) {
  global $user;
  $invitation = temporary_invitation_load($invited_uid);

  // safety check: only the user possessing the invitation may delete it
  if ($invitation !== FALSE
      && $invitation->host_uid == $host_uid
      && ($user->uid == $host_uid && user_access('manage own temporary invitations')
          || user_access('manage temporary invitations')))
  {
    loginticket_delete($invitation->ticket);
  }
  else {
    watchdog('user',
      t('Detected malicious attempt to delete an invitation.'),
      WATCHDOG_WARNING,
      l(t('view'), 'user/'. $user->uid)
    );
  }

  if (isset($goto_path)) {
    drupal_goto($goto_path);
  }
}



/**
 * Implementation of hook_user():
 * If a host user is deleted, also do away with the associated invitations.
 * hook_temporary_invitation($op='delete', $invitation, NULL) is called
 * for each deleted invitation, before the delete action is performed.
 */
function temporary_invitation_user($op, &$edit, &$account, $category = NULL) {
  if ($op == 'delete') {
    $invitations = temporary_invitation_load_array($account->uid);
    $tickets = array();

    foreach ($invitations as $invitation) {
      $tickets[] = $invitation->ticket;
    }
    // let loginticket_delete() invoke all the cleanup operations by itself
    loginticket_delete($tickets);
  }
}

/**
 * Implementation of hook_loginticket():
 * When a ticket is deleted, also delete the associated user
 * and invitation entry from the database, and call
 * hook_temporary_invitation($op='delete', $invitation, NULL)
 * before performing the actual delete actions.
 */
function temporary_invitation_loginticket($op, $ticket, $passcode) {
  if ($ticket->purpose != 'temporary_invitation') {
    return;
  }

  switch ($op) {
    case 'delete':
      $invitation = temporary_invitation_load($ticket->uid);
      if ($invitation !== FALSE) {
        temporary_invitation_invoke_all('delete', $invitation, NULL);
      }

      // delete the invitation entry
      db_query("DELETE FROM {temporary_invitation} WHERE invited_uid = '%d'",
               $ticket->uid);

      // delete the user if (s)he still exists at all
      $account = user_load(array('uid' => $ticket->uid));

      if ($account !== FALSE) {
        switch (_temporary_invitation_get_delete_action()) {
          case 'block':
            if ($account->status == 1) {
              user_save($account, array('status' => 0));
            }
            break;

          case 'delete':
            // workaround to disable the "Temporary user xyz has been deleted." message
            $messages = drupal_get_messages();

            user_delete(array(), $ticket->uid);

            // write back the old messages
            $_SESSION['messages'] = $messages;
            break;
        }
      }
      return;

    default:
      return;
  }
}



/**
 * Implementation of hook_elements():
 * Register the duration widget with the Forms API and set default values.
 */
function temporary_invitation_elements() {
  $type['duration'] = array(
    // the $form_values elements will be named '#name'_metric and '#name'_value
    '#name' => 'duration',
    '#default_metric' => 'mon',
    '#default_value' => 1,
    '#process' => array('_temporary_invitation_duration_process' => array()),
  );
  return $type;
}

/**
 * The process hook for 'duration' type widgets.
 * Called after defining the form and while building it.
 */
function _temporary_invitation_duration_process($element) {
  $name = $element['#name'];
  $element[$name .'_value'] = array(
    '#type' => 'textfield',
    '#default_value' => $element['#default_value'],
    '#size' => 3,
    '#maxlength' => 2,
    '#required' => TRUE,
  );
  $element[$name .'_metric'] = array(
    '#type' => 'select',
    '#default_value' => $element['#default_metric'],
    '#options' => array(
      'hours' => t('hours'),
      'mday' => t('days'),
      'mon' => t('months'),
    ),
  );
  return $element;
}

function theme_duration($element) {
  // class="container-inline" makes child widgets align horizontally
  return theme('form_element', $element,
    '<div class="container-inline">'. $element['#children']. '</div>'
  );
}

/**
 * Add the timespan of a given duration widget to a given point in time.
 *
 * @param $name
 *   The '#name' property of the duration widget.
 * @param $form_values
 *   The $form_values array of the returned form, as received in form_submit().
 * @param $timestamp
 *   The time to which the duration is added. Defaults to the current time.
 */
function temporary_invitation_duration_widget_add($name, $form_values, $timestamp = NULL) {
  return temporary_invitation_duration_add(
    $form_values[$name .'_metric'], $form_values[$name .'_value'], $timestamp
  );
}

/**
 * Add a timespan to a given point in time.
 *
 * @param $metric
 *   What kind of value should be added to the timestamp. This may be one of
 *   'hours', 'minutes', 'seconds', 'mon', 'mday' or 'year', as used in the
 *   associative array returned by getdate().
 * @param $value
 *   How much of the time metric should be added.
 * @param $timestamp
 *   The time to which the duration is added. Defaults to the current time.
 */
function temporary_invitation_duration_add($metric, $value, $timestamp = NULL) {
  if (!isset($timestamp)) {
    $timestamp = time();
  }
  $date = getdate($timestamp);
  $date[$metric] += $value;

  return mktime(
    $date['hours'], $date['minutes'], $date['seconds'],
    $date['mon'], $date['mday'], $date['year']
  );
}

/**
 * Subtract the timespan of a given duration widget from a given point in time.
 *
 * @param $name
 *   The '#name' property of the duration widget.
 * @param $form_values
 *   The $form_values array of the returned form, as received in form_submit().
 * @param $timestamp
 *   The time from which the duration is subtracted.
 *   Defaults to the current time.
 */
function temporary_invitation_duration_widget_subtract($name, $form_values, $timestamp = NULL) {
  return temporary_invitation_duration_subtract(
    $form_values[$name .'_metric'], $form_values[$name .'_value'], $timestamp
  );
}

/**
 * Subtract a timespan to a given point in time.
 *
 * @param $metric
 *   What kind of value should be subtracted from the timestamp. This may be
 *   one of 'hours', 'minutes', 'seconds', 'mon', 'mday' or 'year', as used
 *   in the associative array returned by getdate().
 * @param $value
 *   How much of the time metric should be subtracted.
 * @param $timestamp
 *   The time from which the duration is subtracted.
 *   Defaults to the current time.
 */
function temporary_invitation_duration_subtract($metric, $value, $timestamp = NULL) {
  if (!isset($timestamp)) {
    $timestamp = time();
  }
  temporary_invitation_duration_add($metric, $value * -1, $timestamp);
}



/**
 * Retrieve the roles that will be assigned to newly invited users.
 */
function _temporary_invitation_get_user_roles_default() {
  return variable_get('temporary_invitation_user_roles_default', array());
}

/**
 * Find out what shall be done with the user belonging to an invitation
 * if the invitation is deleted.
 *
 * @param $which
 *   If set to 'all', an associative array consisting of all available
 *   delete action keys and descriptions is returned.
 *   If set to 'selected', the function returns the currently selected key,
 *   which is either 'delete' or 'block'.
 */
function _temporary_invitation_get_delete_action($which = 'selected') {
  if ($which == 'selected') {
    return variable_get('temporary_invitation_delete_action', 'delete');
  }
  else if ($which == 'all') {
    return array(
      'delete' => t('Delete the associated user'),
      'block' => t('Block the associated user'),
    );
  }
}
